/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.console.commands;

import java.rmi.Naming;
import java.util.StringTokenizer;

import edu.umass.cs.geni.dicloud.accounting.AWSAccounting;
import edu.umass.cs.geni.dicloud.console.ConsoleModule;
import edu.umass.cs.geni.dicloud.server.AWSAccoutingInterface;

/**
 * Add credit to the AWS account balance
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class AddCredit extends ConsoleCommand
{

  /**
   * @param module
   */
  public AddCredit(ConsoleModule module)
  {
	super(module);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandDescription
   * ()
   */
  @Override
  public String getCommandDescription()
  {
	return "Add credit to the AWS account";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandName()
   */
  @Override
  public String getCommandName()
  {
	return "add credit";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandParameters
   * ()
   */
  @Override
  public String getCommandParameters()
  {
	return "amount_of_money";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#parse(java.lang
   * .String)
   */
  @Override
  public void parse(String commandText) throws Exception
  {
	StringTokenizer st = new StringTokenizer(commandText);
	if (st.countTokens() != 1)
	{
	  console.printString("Invalid command line arguments\n");
	  return;
	}
	float credit = Float.valueOf(st.nextToken());
	String url = module.getServerUrl() + "/"
	    + AWSAccounting.RMI_NAME_REGISTRATION;
	try
	{
	  AWSAccoutingInterface server = (AWSAccoutingInterface) Naming.lookup(url);
	  console.printString("Current balance is: $" + server.getBalance() + "\n");
	  server.addCredit(credit);
	  console.printString("New balance is: $" + server.getBalance() + "\n");
	}
	catch (Exception e)
	{
	  console.printString("Failed to contact server: " + e + "\n");
	}
  }
}
