/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.console.commands;

import java.rmi.Naming;
import java.sql.Timestamp;
import java.util.List;
import java.util.StringTokenizer;

import edu.umass.cs.geni.dicloud.accounting.AWSAccounting;
import edu.umass.cs.geni.dicloud.accounting.AWSActivity;
import edu.umass.cs.geni.dicloud.console.ConsoleModule;
import edu.umass.cs.geni.dicloud.server.AWSAccoutingInterface;

/**
 * Get the billing history of Amazon Web Services recorded by DiCloud
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class BillingHistory extends ConsoleCommand
{

  /**
   * @param module
   */
  public BillingHistory(ConsoleModule module)
  {
	super(module);
	// TODO Auto-generated constructor stub
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandDescription
   * ()
   */
  @Override
  public String getCommandDescription()
  {
	return "Get the AWS account activity history in CSV or HTML format (a history of 0 retrieves the entire history)";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandName()
   */
  @Override
  public String getCommandName()
  {
	return "billing history";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandParameters
   * ()
   */
  @Override
  public String getCommandParameters()
  {
	return "history_size csv|html";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#parse(java.lang
   * .String)
   */
  @Override
  public void parse(String commandText) throws Exception
  {
	StringTokenizer st = new StringTokenizer(commandText);
	if (st.countTokens() != 2)
	{
	  console.printString("Invalid command line arguments\n");
	  return;
	}
	int historySize = Integer.valueOf(st.nextToken());
	String format = st.nextToken();
	String url = module.getServerUrl() + "/"
	    + AWSAccounting.RMI_NAME_REGISTRATION;
	try
	{
	  AWSAccoutingInterface server = (AWSAccoutingInterface) Naming.lookup(url);
	  List<AWSActivity> list = server.getHistory(historySize);

	  if ("csv".equalsIgnoreCase(format))
	  {
		console.printString("Timestamp,Service,Operation,Cost\n");
		for (AWSActivity awsActivity : list)
		{
		  console.printString(new Timestamp(awsActivity.getTime()) + ","
			  + awsActivity.getService() + "," + awsActivity.getOperation()
			  + "," + awsActivity.getCost() + "\n");
		}
	  }
	  else
	  // HTML
	  {
		console
		    .printString("<table border=\"1\"><tr><th>Timestamp</th><th>Service</th><th>Operation</th><th>Cost</th></tr>\n");
		for (AWSActivity awsActivity : list)
		{
		  console.printString("<tr><td>" + new Timestamp(awsActivity.getTime())
			  + "</td><td>" + awsActivity.getService() + "</td><td>"
			  + awsActivity.getOperation() + "</td><td>"
			  + awsActivity.getCost() + "</td></tr>\n");
		}
		console.printString("</table>\n");
	  }

	}
	catch (Exception e)
	{
	  console.printString("Failed to retrieve information from server: " + e
		  + "\n");
	}
  }
}
