/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.console.commands;

import java.rmi.Naming;
import java.util.StringTokenizer;

import edu.umass.cs.geni.dicloud.console.ConsoleModule;
import edu.umass.cs.geni.dicloud.server.DiCloudServer;

/**
 * Command that connects to the DiCloud server
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class Connect extends ConsoleCommand
{

  /**
   * @param reader
   */
  public Connect(ConsoleModule module)
  {
	super(module);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandDescription
   * ()
   */
  @Override
  public String getCommandDescription()
  {
	return "Connects to the DiCloud server";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandName()
   */
  @Override
  public String getCommandName()
  {
	return "connect";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandParameters
   * ()
   */
  @Override
  public String getCommandParameters()
  {
	return "DiCloudServerHostName RmiPortNumber";
  }

  /**
   * Override execute to not check for existing connectivity
   */
  public void execute(String commandText) throws Exception
  {
	parse(commandText);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#parse(java.lang
   * .String)
   */
  @Override
  public void parse(String commandText) throws Exception
  {
	StringTokenizer st = new StringTokenizer(commandText);
	String url = "//" + st.nextToken() + ":" + st.nextToken();
	module.setServerUrl(url);
	module.setPromptString("dicloud - " + url + ">");
	url = url + "/" + DiCloudServer.RMI_NAME_REGISTRATION;
	if (!module.silent)
	  console.printString("Connecting to " + url + "\n");

	try
	{
	  Naming.lookup(url);
	}
	catch (Exception e)
	{
	  console.printString("Failed to connect to server " + url);
	  module.setServerUrl(null);
	  module.setPromptString("dicloud - not connected>");
	}
  }
}
