/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.console.commands;

import java.rmi.Naming;
import java.util.Properties;
import java.util.StringTokenizer;

import edu.umass.cs.geni.dicloud.AmazonProperties;
import edu.umass.cs.geni.dicloud.console.ConsoleModule;
import edu.umass.cs.geni.dicloud.server.EBSInterface;
import edu.umass.cs.geni.dicloud.server.EBSServer;

/**
 * Delete an EBS volume
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class EBSDelete extends ConsoleCommand
{

  /**
   * @param module
   */
  public EBSDelete(ConsoleModule module)
  {
	super(module);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandDescription
   * ()
   */
  @Override
  public String getCommandDescription()
  {
	return "Delete an EBS volume";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandName()
   */
  @Override
  public String getCommandName()
  {
	return "ebs delete";
  }

  @Override
  public String getCommandParameters()
  {
	return "cert.pem pk.pem volume_id ec2_region";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#parse(java.lang
   * .String)
   */
  @Override
  public void parse(String commandText) throws Exception
  {
	StringTokenizer st = new StringTokenizer(commandText);

	if (st.countTokens() != 4)
	{
	  console.printString("Invalid number of parameters for this command");
	  return;
	}

	// Delete EBS Volume
	Properties p = new Properties();
	p.put(AmazonProperties.AWS_CERT_FILE, st.nextToken());
	p.put(AmazonProperties.AWS_PRIVATE_KEY_FILE, st.nextToken());
	p.put(AmazonProperties.EBS_VOLUME_ID, st.nextToken());
	p.put(AmazonProperties.EC2_REGION, st.nextToken());

	EBSInterface ebsServer = (EBSInterface) Naming.lookup(module.getServerUrl()
	    + "/" + EBSServer.RMI_NAME_REGISTRATION);
	try
	{
	  ebsServer.delete(p);
	}
	catch (Exception e)
	{
	  console.printString("EBS delete failed: " + e.getMessage());
	}
  }

}
