/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.console.commands;

import java.rmi.Naming;
import java.util.Properties;
import java.util.StringTokenizer;

import edu.umass.cs.geni.dicloud.AmazonProperties;
import edu.umass.cs.geni.dicloud.console.ConsoleModule;
import edu.umass.cs.geni.dicloud.server.EC2Interface;
import edu.umass.cs.geni.dicloud.server.EC2Server;

/**
 * Start a new EC2 instance
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class EC2StartInstance extends ConsoleCommand
{

  /**
   * @param module
   */
  public EC2StartInstance(ConsoleModule module)
  {
	super(module);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandDescription
   * ()
   */
  @Override
  public String getCommandDescription()
  {
	return "Start a new EC2 instance";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandName()
   */
  @Override
  public String getCommandName()
  {
	return "ec2 start";
  }

  @Override
  public String getCommandParameters()
  {
	return "cert.pem pk.pem ami_id instance_nb instance_type ec2_region ec2_keypair_name";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#parse(java.lang
   * .String)
   */
  @Override
  public void parse(String commandText) throws Exception
  {
	StringTokenizer st = new StringTokenizer(commandText);

	if (st.countTokens() != 7)
	{
	  console.printString("Invalid number of parameters for this command");
	  return;
	}

	// Create an EC2 instance
	Properties p = new Properties();
	p.put(AmazonProperties.AWS_CERT_FILE, st.nextToken());
	p.put(AmazonProperties.AWS_PRIVATE_KEY_FILE, st.nextToken());
	p.put(AmazonProperties.EC2_AMI_ID, st.nextToken());
	p.put(AmazonProperties.EC2_INSTANCE_NUMBER, st.nextToken());
	p.put(AmazonProperties.EC2_INSTANCE_TYPE, st.nextToken());
	p.put(AmazonProperties.EC2_REGION, st.nextToken());
	// p.put(AmazonProperties.EC2_AVAILABILITY_ZONE, st.nextToken());
	p.put(AmazonProperties.EC2_KEY_PAIR_NAME, st.nextToken());

	// Launch the instance
	EC2Interface ec2Server = (EC2Interface) Naming.lookup(module.getServerUrl()
	    + "/" + EC2Server.RMI_NAME_REGISTRATION);
	try
	{
	  Properties result = ec2Server.startInstance(p);
	  // Probe instance
	  p.put(AmazonProperties.EC2_INSTANCE_ID, result
		  .get(AmazonProperties.EC2_INSTANCE_ID));
	  ec2Server.describeInstance(p);
	  console.printString((String) p.get(AmazonProperties.EC2_INSTANCE_ID));
	}
	catch (Exception e)
	{
	  console.printString("Failed to start EC2 instance: " + e.getMessage());
	}
  }

}
