/**
 * Sequoia: Database clustering technology.
 * Copyright (C) 2002-2004 French National Institute For Research In Computer
 * Science And Control (INRIA).
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 *
 * Initial developer(s): Nicolas Modrzyk.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.console.commands;

import java.util.List;
import java.util.StringTokenizer;

import edu.umass.cs.geni.dicloud.console.ConsoleModule;

/**
 * This class defines a History
 * 
 * @author <a href="mailto:Nicolas.Modrzyk@inrialpes.fr">Nicolas Modrzyk </a>
 * @version 1.0
 */
public class History extends ConsoleCommand
{

  /**
   * Creates a new <code>History</code> object
   * 
   * @param module
   *          module that owns this commands
   */
  public History(ConsoleModule module)
  {
	super(module);
  }

  /**
   * @see org.continuent.sequoia.console.text.commands.ConsoleCommand#parse(java.lang.String)
   */
  public void parse(String commandText) throws Exception
  {
	List list = module.getHistory();
	StringTokenizer st = new StringTokenizer(commandText);
	if (st.countTokens() == 0)
	{
	  for (int i = 0; i < list.size(); i++)
	  {
		Object o = list.get(i);
		console.printString("" + i + "\t" + o + "\n"); //$NON-NLS-1$ //$NON-NLS-2$
	  }
	}
	else
	{
	  int line = Integer.parseInt(st.nextToken());
	  console.printString((String) list.get(line) + "\n");
	  module.handleCommandLine((String) list.get(line), module
		  .getHashCommands());
	}
  }

  /**
   * @see org.continuent.sequoia.console.text.commands.ConsoleCommand#getCommandName()
   */
  public String getCommandName()
  {
	return "history"; //$NON-NLS-1$
  }

  /**
   * @see org.continuent.sequoia.console.text.commands.ConsoleCommand#getCommandDescription()
   */
  public String getCommandDescription()
  {
	return "Display history of commands for the console.";
  }

  /**
   * @see org.continuent.sequoia.console.text.commands.ConsoleCommand#getCommandParameters()
   */
  public String getCommandParameters()
  {
	return "[<command index>]"; //$NON-NLS-1$
  }
}