/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.console.commands;

import java.io.File;
import java.io.FileInputStream;
import java.rmi.Naming;
import java.util.StringTokenizer;

import edu.umass.cs.geni.dicloud.console.ConsoleModule;
import edu.umass.cs.geni.dicloud.server.S3ProxyInterface;

/**
 * Put an object in an S3 bucket
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class S3Put extends ConsoleCommand
{

  /**
   * @param module
   */
  public S3Put(ConsoleModule module)
  {
	super(module);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandDescription
   * ()
   */
  @Override
  public String getCommandDescription()
  {
	return "Put the content of the specified file in an object named object_name in the S3 bucket";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandName()
   */
  @Override
  public String getCommandName()
  {
	return "s3 put";
  }

  @Override
  public String getCommandParameters()
  {
	return "s3_bucket_name object_name file_path";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#parse(java.lang
   * .String)
   */
  @Override
  public void parse(String commandText) throws Exception
  {
	StringTokenizer st = new StringTokenizer(commandText);

	if (st.countTokens() != 3)
	{
	  console.printString("Invalid number of parameters for this command");
	  return;
	}

	byte[] data;
	String bucketName = st.nextToken();
	String objectName = st.nextToken();
	String fileName = st.nextToken();

	S3ProxyInterface s3Proxy = (S3ProxyInterface) Naming.lookup(module
	    .getServerUrl()
	    + "/" + bucketName);

	console.printString("Uploading file " + fileName + " as object "
	    + objectName + " in bucket " + bucketName);
	File f = new File(fileName);
	int size = (int) f.length();
	data = new byte[size];
	FileInputStream fis = new FileInputStream(f);
	fis.read(data);
	fis.close();

	try
	{
	  s3Proxy.put(objectName, data);
	}
	catch (Exception e)
	{
	  console.printString("Failed to upload object: " + e.getMessage());
	}
  }

}
