/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.console.commands;

import java.rmi.Naming;

import edu.umass.cs.geni.dicloud.console.ConsoleModule;
import edu.umass.cs.geni.dicloud.server.DiCloudServer;
import edu.umass.cs.geni.dicloud.server.DiCloudServerInterface;

/**
 * Used to shutdown the DiCloud server (and properly terminate the Derby
 * database)
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class ShutdownServer extends ConsoleCommand
{

  /**
   * 
   * @param module
   */
  public ShutdownServer(ConsoleModule module)
  {
	super(module);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandDescription
   * ()
   */
  @Override
  public String getCommandDescription()
  {
	return "Shutdown the DiCloud server the console is currently connected to.";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandName()
   */
  @Override
  public String getCommandName()
  {
	return "shutdown";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#getCommandParameters
   * ()
   */
  @Override
  public String getCommandParameters()
  {
	return "";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * edu.umass.cs.geni.dicloud.console.commands.ConsoleCommand#parse(java.lang
   * .String)
   */
  @Override
  public void parse(String commandText) throws Exception
  {
	if (module.getServerUrl() == null)
	{
	  console
		  .printString("Not connected to a DiCloud server. Cannot shutdown.\n");
	  return;
	}

	module.getConsole().printString(
	    "Shutting down server " + module.getServerUrl() + "\n");
	String url = module.getServerUrl() + "/"
	    + DiCloudServer.RMI_NAME_REGISTRATION;
	try
	{
	  DiCloudServerInterface server = (DiCloudServerInterface) Naming
		  .lookup(url);
	  server.shutdown();
	}
	catch (Exception e)
	{
	  console.printString("Failed to shutdown server (are you connected?): "
		  + e + "\n");
	}
  }

}
