/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.handlers;

import java.util.Properties;

import org.apache.log4j.Logger;

import edu.umass.cs.geni.dicloud.AmazonProperties;

import orca.shirako.plugins.ShirakoPlugin;
import orca.shirako.plugins.config.Config;

/**
 * Tests the EC2 Handler
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class TestEC2Handler
{

  /**
   * @param args
   */
  public static void main(String[] args)
  {
	Properties p = new Properties();
	p.put(AmazonProperties.AWS_CERT_FILE,
	    "/home/manu/ec2/keys/cert-PKPHQWFUX6DU7GRIRPMWXVGN6SYYNI3B.pem");
	p.put(AmazonProperties.AWS_PRIVATE_KEY_FILE,
	    "/home/manu/ec2/keys/pk-PKPHQWFUX6DU7GRIRPMWXVGN6SYYNI3B.pem");
	p.put(AmazonProperties.EC2_AMI_ID, "ami-eccf2285");
	p.put(AmazonProperties.EC2_INSTANCE_NUMBER, "1");
	p.put(AmazonProperties.EC2_INSTANCE_TYPE, "m1.small");
	p.put(AmazonProperties.EC2_REGION, "us-east-1");
	p.put(AmazonProperties.EC2_AVAILABILITY_ZONE, "us-east-1a");

	try
	{
	  AWSPlugin plugin = new AWSPlugin();
	  plugin.setLogger(Logger.getRootLogger());
	  plugin.initialize();
	  EC2Handler handler = new EC2Handler(plugin);
	  handler.initialize();

	  // Create instance
	  ec2_join(p, plugin, handler);

	  // Probe instance
	  p.put(AmazonProperties.EC2_INSTANCE_ID, plugin
		  .getConfigurationProperties().get(AmazonProperties.EC2_INSTANCE_ID));
	  for (int i = 0; i < 6 * 5; i++)
	  {
		ec2_probe(p, plugin, handler);
		Thread.sleep(10000);
	  }

	  // Delete instance
	  p
		  .put(AmazonProperties.EC2_KEY_PAIR_NAME, plugin
		      .getConfigurationProperties().get(
		          AmazonProperties.EC2_KEY_PAIR_NAME));
	  ec2_leave(p, plugin, handler);
	}
	catch (Exception e)
	{
	  // TODO Auto-generated catch block
	  e.printStackTrace();
	}
  }

  private static void ec2_join(Properties p, ShirakoPlugin plugin,
	  EC2Handler handler) throws Exception
  {
	handler.join(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	System.out.println("Join result code: " + result);
	System.out.println("AMI status: "
	    + props.getProperty(Config.PropertyTargetResultCodeMessage));
	System.out.println("Error code:"
	    + props.getProperty(Config.PropertyExceptionMessage));
	System.out.println("Error stack:"
	    + props.getProperty(Config.PropertyExceptionStack));

	if (!"0".equals(result))
	  throw new Exception("Failed to create EC2 instance");
  }

  private static void ec2_probe(Properties p, ShirakoPlugin plugin,
	  EC2Handler handler) throws Exception
  {
	handler.probe(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	System.out.println("Probing result code: " + result);

	if ("0".equals(result))
	{
	  System.out.println("AMI status: "
		  + props.getProperty(Config.PropertyTargetResultCodeMessage));
	  System.out.println("Instance id: "
		  + props.getProperty(AmazonProperties.EC2_INSTANCE_ID));
	  System.out.println("Public name: "
		  + props.getProperty(AmazonProperties.EC2_PUBLIC_NAME));
	  System.out.println("Private name: "
		  + props.getProperty(AmazonProperties.EC2_PRIVATE_NAME));
	  System.out.println("Public IP: "
		  + props.getProperty(AmazonProperties.EC2_PUBLIC_IP));
	  System.out.println("Private IP: "
		  + props.getProperty(AmazonProperties.EC2_PRIVATE_IP));
	  System.out.println("Instance state: "
		  + props.getProperty(AmazonProperties.EC2_INSTANCE_STATE));
	  System.out.println("Key pair: "
		  + props.getProperty(AmazonProperties.EC2_KEY_PAIR_NAME));
	  System.out.println("Start time: "
		  + props.getProperty(AmazonProperties.EC2_START_TIME));
	  System.out.println("Availability zone: "
		  + props.getProperty(AmazonProperties.EC2_AVAILABILITY_ZONE));
	}
	else
	{
	  System.out.println("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  System.out.println("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to probe EC2 instance");
	}
  }

  private static void ec2_leave(Properties p, ShirakoPlugin plugin,
	  EC2Handler handler) throws Exception
  {
	handler.leave(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	System.out.println("Leave result code: " + result);
	System.out.println("Status: "
	    + props.getProperty(Config.PropertyTargetResultCodeMessage));
	System.out.println("Error code:"
	    + props.getProperty(Config.PropertyExceptionMessage));
	System.out.println("Error stack:"
	    + props.getProperty(Config.PropertyExceptionStack));

	if (!"0".equals(result))
	  throw new Exception("Failed to terminate EC2 instance");
  }

}
