/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.handlers;

import java.util.Properties;

import orca.shirako.plugins.ShirakoPlugin;
import orca.shirako.plugins.config.Config;

import org.apache.log4j.Logger;

import edu.umass.cs.geni.dicloud.AmazonProperties;

/**
 * Tests the S3 Handler
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class TestS3Handler
{

  /**
   * @param args
   */
  public static void main(String[] args)
  {
	AWSPlugin plugin = null;
	S3Handler handler = null;
	Properties p = null;
	try
	{
	  plugin = new AWSPlugin();
	  plugin.setLogger(Logger.getRootLogger());
	  plugin.initialize();
	  handler = new S3Handler(plugin);
	  handler.initialize();
	  p = new Properties();
	}
	catch (Exception e1)
	{
	  System.out.println("Failed to initialize handler/plugin");
	  e1.printStackTrace();
	  System.exit(-1);
	}

	try
	{
	  p.put(AmazonProperties.S3_BUCKET_NAME, "s3://geni-dicloud-s3handlertest");

	  // Create bucket
	  s3_join(p, plugin, handler);

	  // Probe bucket
	  for (int i = 0; i < 3; i++)
	  {
		s3_probe(p, plugin, handler);
		Thread.sleep(10000);
	  }

	  // Delete bucket
	  s3_leave(p, plugin, handler);
	}
	catch (Exception e)
	{
	  // TODO Auto-generated catch block
	  e.printStackTrace();
	}

	try
	{
	  p
		  .put(AmazonProperties.S3_BUCKET_NAME,
		      "s3://NonExistingBucketShouldFail");
	  s3_probe(p, plugin, handler);
	  System.out
		  .println("FAILED: Did not get an error on non existing S3 bucket");
	}
	catch (Exception ignore)
	{
	}
  }

  private static void s3_join(Properties p, ShirakoPlugin plugin,
	  S3Handler handler) throws Exception
  {
	handler.join(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	System.out.println("Join result code: " + result);
	System.out.println("command status: "
	    + props.getProperty(Config.PropertyTargetResultCodeMessage));
	System.out.println("Error code:"
	    + props.getProperty(Config.PropertyExceptionMessage));
	System.out.println("Error stack:"
	    + props.getProperty(Config.PropertyExceptionStack));
  }

  private static void s3_probe(Properties p, ShirakoPlugin plugin,
	  S3Handler handler) throws Exception
  {
	handler.probe(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	System.out.println("Probing result code: " + result);

	if ("0".equals(result))
	{
	  System.out.println("command status: "
		  + props.getProperty(Config.PropertyTargetResultCodeMessage));
	}
	else
	{
	  System.out.println("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  System.out.println("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to probe s3 bucket");
	}
  }

  private static void s3_leave(Properties p, ShirakoPlugin plugin,
	  S3Handler handler) throws Exception
  {
	handler.leave(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	System.out.println("Leave result code: " + result);
	System.out.println("Status: "
	    + props.getProperty(Config.PropertyTargetResultCodeMessage));
	System.out.println("Error code:"
	    + props.getProperty(Config.PropertyExceptionMessage));
	System.out.println("Error stack:"
	    + props.getProperty(Config.PropertyExceptionStack));

	if (!"0".equals(result))
	  throw new Exception("Failed to delete s3 bucket");
  }

}
