/**
 * DiCloud - GENI project
 * 
 * (C) 2010 - University of Massachusetts Amherst
 * 
 * Initial developer(s): Emmanuel Cecchet.
 * Contributor(s): ______________________.
 */

package edu.umass.cs.geni.dicloud.test;

import java.util.Properties;

import orca.shirako.plugins.ShirakoPlugin;
import orca.shirako.plugins.config.Config;

import org.apache.log4j.Logger;

import edu.umass.cs.geni.dicloud.AmazonProperties;
import edu.umass.cs.geni.dicloud.accounting.AWSAccounting;
import edu.umass.cs.geni.dicloud.accounting.AWSActivity;
import edu.umass.cs.geni.dicloud.handlers.AWSPlugin;
import edu.umass.cs.geni.dicloud.handlers.EC2Handler;
import edu.umass.cs.geni.dicloud.handlers.S3Handler;

/**
 * Test the AWS accounting service
 * 
 * @author <a href="mailto:cecchet@cs.umass.edu">Emmanuel Cecchet </a>
 * @version 1.0
 * 
 */
public class AWSAccountingTest
{

  /**
   * @param args
   *          1. initial credit, 2. minimum credit 3.optional RMI port number
   */
  public static void main(String[] args)
  {
	AWSPlugin plugin = null;
	EC2Handler ec2Handler = null;
	Properties p = null;
	AWSAccounting awsAccounting = null;
	try
	{
	  if (args.length < 2)
	  {
		System.out
		    .println("Usage: java -cp ... edu.umass.cs.geni.dicloud.test.AWSAccountingTest initial_credit minimum_credit [RMI_port_number]");
		System.exit(1);
	  }

	  // Start the RMI registry for the S3 Proxy
	  int rmiPort = 1099;
	  if (args.length > 2)
		rmiPort = Integer.valueOf(args[2]);
	  java.rmi.registry.LocateRegistry.createRegistry(rmiPort);

	  // Create the AWS accounting service
	  plugin = new AWSPlugin();
	  plugin.setLogger(Logger.getRootLogger());
	  plugin.initialize();

	  awsAccounting = plugin.getAwsAccouting();
	  awsAccounting.resetDatabase();
	  awsAccounting.setMinimumCredit(Double.valueOf(args[1]));
	  awsAccounting.logActivity(new AWSActivity("Payment", "Credit " + args[0],
		  Double.valueOf(args[0]), System.currentTimeMillis()));

	  ec2Handler = new EC2Handler(plugin);
	  ec2Handler.initialize();
	  S3Handler s3Handler = new S3Handler(plugin);
	  s3Handler.initialize();

	  // Create an EC2 instance
	  p = new Properties();
	  p.put(AmazonProperties.AWS_CERT_FILE,
		  "/home/cecchet/ec2/keys/cert-PKPHQWFUX6DU7GRIRPMWXVGN6SYYNI3B.pem");
	  p.put(AmazonProperties.AWS_PRIVATE_KEY_FILE,
		  "/home/cecchet/ec2/keys/pk-PKPHQWFUX6DU7GRIRPMWXVGN6SYYNI3B.pem");
	  p.put(AmazonProperties.EC2_AMI_ID, "ami-78f21911");
	  p.put(AmazonProperties.EC2_INSTANCE_NUMBER, "1");
	  p.put(AmazonProperties.EC2_INSTANCE_TYPE, "m1.small");
	  p.put(AmazonProperties.EC2_REGION, "us-east-1");
	  p.put(AmazonProperties.EC2_AVAILABILITY_ZONE, "us-east-1a");

	  // Launch the instance
	  ec2_join(p, plugin, ec2Handler);

	  // Probe instance
	  p.put(AmazonProperties.EC2_INSTANCE_ID, plugin
		  .getConfigurationProperties().get(AmazonProperties.EC2_INSTANCE_ID));

	  // Create S3 bucket
	  p.put(AmazonProperties.S3_BUCKET_NAME, "geni-dicloud-gec8demo");
	  s3_join(p, plugin, s3Handler);

	  // Poll until the instance is ready
	  while (!ec2_probe(p, plugin, ec2Handler))
		Thread.sleep(10000);

	  System.out.println("Connect to the instance. Sleeping for 120 minutes");
	  Thread.sleep(120 * 60 * 1000);

	  // Delete bucket
	  s3_leave(p, plugin, s3Handler);

	  try
	  {
		// Delete EC2 instance
		p.put(AmazonProperties.EC2_KEY_PAIR_NAME, plugin
		    .getConfigurationProperties().get(
		        AmazonProperties.EC2_KEY_PAIR_NAME));
		ec2_leave(p, plugin, ec2Handler);
	  }
	  catch (Exception e)
	  {
		System.err.println("Failed to free EC2 instance " + e);
	  }
	}
	catch (Exception e)
	{
	  // TODO Auto-generated catch block
	  e.printStackTrace();
	}
	finally
	{
	  awsAccounting.shutdown();
	}
  }

  private static void ec2_join(Properties p, ShirakoPlugin plugin,
	  EC2Handler handler) throws Exception
  {
	handler.join(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	if ("0".equals(result))
	{
	  System.out.println("Key pair: "
		  + props.getProperty(AmazonProperties.EC2_KEY_PAIR_NAME) + ".pem\n"
		  + props.getProperty(AmazonProperties.EC2_KEY_PAIR) + "\n");
	  System.out.println("AMI status: "
		  + props.getProperty(Config.PropertyTargetResultCodeMessage));
	}
	else
	{
	  System.out.println("EC2 Join result code: " + result);
	  System.out.println("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  System.out.println("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to create EC2 instance");
	}
  }

  /**
   * 
   * @param p
   * @param plugin
   * @param handler
   * @return true if the EC2 instance is ready in running state
   * @throws Exception
   */
  private static boolean ec2_probe(Properties p, ShirakoPlugin plugin,
	  EC2Handler handler) throws Exception
  {
	handler.probe(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	String status = null;

	if ("0".equals(result))
	{
	  status = props.getProperty(Config.PropertyTargetResultCodeMessage);
	  System.out.println("AMI status: " + status);
	  System.out.println("Instance id: "
		  + props.getProperty(AmazonProperties.EC2_INSTANCE_ID));
	  System.out.println("AMI id: "
		  + props.getProperty(AmazonProperties.EC2_AMI_ID));
	  System.out.println("Public name: "
		  + props.getProperty(AmazonProperties.EC2_PUBLIC_NAME));
	  System.out.println("Private name: "
		  + props.getProperty(AmazonProperties.EC2_PRIVATE_NAME));
	  System.out.println("Public IP: "
		  + props.getProperty(AmazonProperties.EC2_PUBLIC_IP));
	  System.out.println("Private IP: "
		  + props.getProperty(AmazonProperties.EC2_PRIVATE_IP));
	  System.out.println("Instance state: "
		  + props.getProperty(AmazonProperties.EC2_INSTANCE_STATE));
	  System.out.println("Instance type: "
		  + props.getProperty(AmazonProperties.EC2_INSTANCE_TYPE));
	  System.out.println("Key pair: "
		  + props.getProperty(AmazonProperties.EC2_KEY_PAIR_NAME));
	  System.out.println("Start time: "
		  + props.getProperty(AmazonProperties.EC2_START_TIME));
	  System.out.println("Availability zone: "
		  + props.getProperty(AmazonProperties.EC2_AVAILABILITY_ZONE));
	}
	else
	{
	  System.out.println("EC2 Probing result code: " + result);
	  System.out.println("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  System.out.println("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to probe EC2 instance");
	}
	return (status != null) && status.contains("running");
  }

  private static void ec2_leave(Properties p, ShirakoPlugin plugin,
	  EC2Handler handler) throws Exception
  {
	handler.leave(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	if (!"0".equals(result))
	{
	  System.out.println("EC2 Leave result code: " + result);
	  System.out.println("Status: "
		  + props.getProperty(Config.PropertyTargetResultCodeMessage));
	  System.out.println("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  System.out.println("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to terminate EC2 instance");
	}
  }

  private static void s3_join(Properties p, ShirakoPlugin plugin,
	  S3Handler handler) throws Exception
  {
	handler.join(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	if (!"0".equals(result))
	{
	  System.out.println("S3 Join result code: " + result);
	  System.out.println("command status: "
		  + props.getProperty(Config.PropertyTargetResultCodeMessage));
	  System.out.println("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  System.out.println("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	}
  }

  private static void s3_leave(Properties p, ShirakoPlugin plugin,
	  S3Handler handler) throws Exception
  {
	handler.leave(null, p);

	Properties props = plugin.getConfigurationProperties();
	String result = props.getProperty(Config.PropertyTargetResultCode);
	if ("0".equals(result))
	{
	  System.out.println("S3 Leave result code: " + result);
	  System.out.println("Status: "
		  + props.getProperty(Config.PropertyTargetResultCodeMessage));
	  System.out.println("Error code:"
		  + props.getProperty(Config.PropertyExceptionMessage));
	  System.out.println("Error stack:"
		  + props.getProperty(Config.PropertyExceptionStack));
	  throw new Exception("Failed to delete s3 bucket");
	}
  }

}
